//#define ERROR_CHECK_LEVEL = ERROR_CHECK_FULL
/***********************************************************************
 *
 *	Copyright  Ben Wagner 2000 -- All Rights Reserved
 *
 * PROJECT: Counter
 *
 * FILE: MemoPad.c (Phase 4)
 *
 *       For best formatting set tab size to 3.
 *
 * DESCRIPTION: A simple counting program.
 *
 *	OBJECTIVE: Finish cleaning up code.
 *
 * REVISION HISTORY:
 *
 **********************************************************************/

#include <Pilot.h>				// all the system toolbox headers
#include "MemoPadRsc.h"			// application resource defines

typedef struct {
	Int32	increment;
	Int32	currentNumber;
} CounterPreferenceType;


/***********************************************************************
 * Prototypes for internal functions
 **********************************************************************/
static void StartApplication(void);
static void StopApplication(void);
static Boolean MainFormHandleEvent(EventPtr event);
static void EventLoop(void);
void CounterSavePrefs(void);
void CounterLoadPrefs(void);

Int32 increment = 1, currentNumber = 0;

/***********************************************************************
 *
 * FUNCTION:    CounterSavePrefs
 *
 * DESCRIPTION: Save the Counter preferences
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/

void CounterSavePrefs(void)
{
	CounterPreferenceType prefs;

	// Write the preferences / saved-state information.
	prefs.increment = increment;
	prefs.currentNumber = currentNumber;
	   
	// Write the state information.
	PrefSetAppPreferences (1, 0, 1, &prefs, 
		sizeof (CounterPreferenceType), true);
}

/***********************************************************************
 *
 * FUNCTION:    CounterLoadPrefs
 *
 * DESCRIPTION: Load the application preferences
 *
 * PARAMETERS:  none
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/

void CounterLoadPrefs(void)
{
	Int prefsRet;
	UInt prefsSize;
	CounterPreferenceType prefs;

	prefsSize = sizeof (CounterPreferenceType);
	prefsRet = PrefGetAppPreferences (1, 0, &prefs, &prefsSize, true);
	
	//if there is no problem get values, otherwise just use globals as is (defaults)
	if (prefsRet <= prefsSize && prefsRet != noPreferenceFound) {
		increment = prefs.increment;
		currentNumber = prefs.currentNumber;
	}
}

/***********************************************************************
 *
 * FUNCTION:     StartApplication
 *
 * DESCRIPTION:  This routine sets up the initial state of the application.
 *
 * PARAMETERS:   None.
 *
 * RETURNED:     Nothing.
 *
 ***********************************************************************/
static void StartApplication(void)
{
	CounterLoadPrefs();
	
	// Initialize and draw the main memo pad form.
	FrmGotoForm(CounterMainForm);
}

static void StopApplication(void)
{
	FrmCloseAllForms();
	CounterSavePrefs();
}

/***********************************************************************
 *
 * FUNCTION:		MainFormHandleEvent
 *
 * DESCRIPTION:	Handles processing of events for the main form.
 *
 * PARAMETERS:		event	- the most recent event.
 *
 * RETURNED:		True if the event is handled, false otherwise.
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr event)
{
	Boolean	handled = false;
	//static Handle newLabelH = NULL;
	static Char *newLabel = NULL;
	FormPtr	frm = NULL;
	Handle clip = NULL;
	Ptr clipPtr = NULL;
	UInt16 len;

	switch (event->eType) {
	case keyDownEvent:
		switch (event->data.keyDown.chr) {
		case pageUpChr:
			//always go up, even if increment is negative
			if (increment > 0)
				currentNumber += increment;
			else
				currentNumber -= increment;
			
			FrmUpdateForm(CounterMainForm, frmRedrawUpdateCode);
			handled = true;
			break;
		case pageDownChr:
			//always go down, even if increment is negative
			if (increment > 0)
				currentNumber -= increment;
			else
				currentNumber += increment;

			FrmUpdateForm(CounterMainForm, frmRedrawUpdateCode);
			handled = true;
			break;
		}
		break;
		
	case ctlSelectEvent:
		switch (event->data.ctlSelect.controlID) {
		case CounterMainCounterButton:
			currentNumber += increment;
			FrmUpdateForm(CounterMainForm, frmRedrawUpdateCode);
			
			handled = true;
			break;
		}
		break;
	
	case frmOpenEvent:
	  	// It has already been loaded and activated so just draw it.
	  	newLabel = MemPtrNew(20);
		
	  	frm = FrmGetActiveForm();
		FrmDrawForm (frm);
		FrmUpdateForm(CounterMainForm, frmRedrawUpdateCode);
		
		handled = true;
		break;
		
	case frmUpdateEvent:
	  	frm = FrmGetFormPtr(CounterMainForm);
	  	
		if (frm != NULL) {
			// Set button to the right number.
			StrIToA(newLabel, currentNumber);
			CtlSetLabel (frm->objects[0].object.control, newLabel);
		}
		handled = true;
		break;
	
	case frmCloseEvent:
		MemPtrFree(newLabel);
		break;
		
	case menuEvent:
			
		// Process menu commands for the counter form.
		switch (event->data.menu.itemID) {
		case RecordSetNumber:
			// First clear the menu status from the display.
			MenuEraseStatus(0);

			//get a number and set currentNumber to it and update display
			FrmPopupForm(SetNumberForm);
			break;

		case EditCopy:
			//put the currentNumber on the clipboard
			ClipboardAddItem (clipboardText, newLabel, StrLen(newLabel));
			break;

		case EditPaste:
			// First clear the menu status from the display.
			MenuEraseStatus(0);

			//convert what's on the clipboard to a number and set currentNumber to it and update the display. If the value is 0 and the text is not "0", then alert "The clipboard does not appear to contain a number. The data is..."
			clip = ClipboardGetItem(clipboardText, &len);
			if (len > 0 && clip != NULL) {
				StrCopy(newLabel, "           ");
				clipPtr = MemHandleLock(clip);
				MemMove (newLabel, clipPtr, len);
				//StrNCopy looks ahead for a '/0' at the end, so it goes 1 byte over and makes errors, even though it's just reading garbage that doesn't make a difference. The len is in bytes, not characters.
        		//StrNCopy (newLabel, clipPtr, len);
				MemHandleUnlock(clip);
				currentNumber = StrAToI(newLabel);
				
				FrmUpdateForm(CounterMainForm, frmRedrawUpdateCode);
			}
			break;
			
		case RecordCountBy:
			//get the number for startNumber and increment
			FrmPopupForm(PrefsForm);
			break;

		case RecordAboutCounter:
			//show about box
			frm = FrmInitForm(AboutForm);
			FrmDoDialog(frm);
	 		FrmDeleteForm(frm);
			break;
		}
		handled = true;
		break;
	}

	return(handled);
}

/***********************************************************************
 *
 * FUNCTION:		SetNumberHandleEvent
 *
 * DESCRIPTION:	Handles processing of events for the SetNumber form.
 *
 * PARAMETERS:		event	- the most recent event.
 *
 * RETURNED:		True if the event is handled, false otherwise.
 *
 ***********************************************************************/
static Boolean SetNumberHandleEvent(EventPtr event)
{
	Boolean	handled = false;
	static Boolean positive = true;
	Int fldIndex, toggleIndex;
	FormType *frm;
	static Handle labelHandle;
	Char *labelPointer;
	Char *neg = "-", *pos = "+";
	
	switch (event->eType) {
	case ctlSelectEvent:
		switch (event->data.ctlSelect.controlID) {
		case SetNumberSetToggleButton:
			frm = FrmGetActiveForm();
			toggleIndex = FrmGetObjectIndex(frm, SetNumberSetToggleButton);
			if (positive) {
				CtlSetLabel (frm->objects[toggleIndex].object.control, neg);
				positive = false;
			} else {
				CtlSetLabel (frm->objects[toggleIndex].object.control, pos);
				positive = true;
			}
			break;
				
		case SetNumberSetOKButton:
			frm = FrmGetActiveForm();

			fldIndex = FrmGetObjectIndex(frm, SetNumberNumberField);
			currentNumber = StrAToI(FldGetTextPtr(FrmGetObjectPtr (frm, fldIndex)));
			if (positive == false)
				currentNumber = -currentNumber;
				
			FrmUpdateForm(CounterMainForm, frmRedrawUpdateCode);
			FrmReturnToForm(CounterMainForm);
			handled = true;
			break;
			
		case SetNumberSetCancelButton:
			FrmReturnToForm(CounterMainForm);
			handled = true;
			break;
		}
		break;
		
	case frmOpenEvent:
		frm = FrmGetActiveForm();

		//set the text field to the absolute value of the current number
		//and set the +/- toggle button
		toggleIndex = FrmGetObjectIndex(frm, SetNumberSetToggleButton);
		fldIndex = FrmGetObjectIndex(frm, SetNumberNumberField);
		labelHandle = MemHandleNew(20);
		labelPointer = MemHandleLock (labelHandle);
		if (currentNumber < 0) {
			CtlSetLabel (frm->objects[toggleIndex].object.control, neg);
			StrIToA(labelPointer, -currentNumber);
			positive = false;
		} else {
			CtlSetLabel (frm->objects[toggleIndex].object.control, pos);
			StrIToA(labelPointer, currentNumber);
			positive = true;
		}
		MemHandleUnlock (labelHandle);
   		FldSetTextHandle(FrmGetObjectPtr(frm, fldIndex), labelHandle);
   		
   		FrmDrawForm(frm);

		// set focus to the edit field
		FrmSetFocus(frm, fldIndex);
		handled = true;
		break;
		
	case frmCloseEvent:
		//this is unessesary, labelHandel will be freed by the field
		//ErrDisplay ("freeing labelHandel in SetNumber");
		//MemHandleFree(labelHandle);
		break;
	}
	return(handled);
}

/***********************************************************************
 *
 * FUNCTION:		PrefrenceHandleEvent
 *
 * DESCRIPTION:	Handles processing of events for the Prefrence form.
 *
 * PARAMETERS:		event	- the most recent event.
 *
 * RETURNED:		True if the event is handled, false otherwise.
 *
 ***********************************************************************/
static Boolean PrefrenceHandleEvent(EventPtr event)
{
	Boolean	handled = false;
	static Boolean incrementPositive = true;

	FormType *frm;
	Int incrementFldIndex, incrementToggleIndex;
	static Handle incrementHandle = NULL; 
	Char *incrementPointer;
	Char *neg = "-", *pos = "+";
	
	switch (event->eType) {
	case ctlSelectEvent:
		switch (event->data.ctlSelect.controlID) {			
		case PrefsIncrementToggleButton:
			frm = FrmGetActiveForm();
			incrementToggleIndex = FrmGetObjectIndex(frm, PrefsIncrementToggleButton);
			if (incrementPositive) {
				CtlSetLabel (frm->objects[incrementToggleIndex].object.control, neg);
				incrementPositive = false;
			} else {
				CtlSetLabel (frm->objects[incrementToggleIndex].object.control, pos);
				incrementPositive = true;
			}
			break;
				
		case PrefsOKButton:
			frm = FrmGetActiveForm();
				
			incrementFldIndex = FrmGetObjectIndex(frm, PrefsIncrementField);
			increment = StrAToI(FldGetTextPtr(FrmGetObjectPtr (frm, incrementFldIndex)));
			if (incrementPositive == false)
				increment = -increment;
				
			FrmReturnToForm(CounterMainForm);
			handled = true;
			break;
			
		case PrefsCancelButton:
			FrmReturnToForm(CounterMainForm);
			handled = true;
			break;
		}
		break;
		
	case frmOpenEvent:
		frm = FrmGetActiveForm();

   		//set the increment number field and the +/- toggle button
		incrementToggleIndex = FrmGetObjectIndex(frm, PrefsIncrementToggleButton);
		incrementFldIndex = FrmGetObjectIndex(frm, PrefsIncrementField);
		incrementHandle = MemHandleNew(20);
		incrementPointer = MemHandleLock (incrementHandle);
		if (increment < 0) {
			CtlSetLabel (frm->objects[incrementToggleIndex].object.control, neg);
			StrIToA(incrementPointer, -increment);
			incrementPositive = false;
		} else {
			CtlSetLabel (frm->objects[incrementToggleIndex].object.control, pos);
			StrIToA(incrementPointer, increment);
			incrementPositive = true;
		}
		MemHandleUnlock (incrementHandle);
   		FldSetTextHandle(FrmGetObjectPtr(frm, incrementFldIndex), incrementHandle);

		FrmDrawForm(frm);

		// set focus to the start number edit field
		FrmSetFocus(frm, incrementFldIndex);
		handled = true;
		break;
		
	case frmCloseEvent:
		//ErrDisplay ("freeing incrementHandle in CountbyForm");
		//MemHandleFree(incrementHandle);
		
		break;
	}
	return(handled);
}

/***********************************************************************
 *
 * FUNCTION:    ApplicationHandleEvent
 *
 * DESCRIPTION: P3. This routine loads a form resource and sets the event handler for the form.
 *
 * PARAMETERS:  event - a pointer to an EventType structure
 *
 * RETURNED:    True if the event has been handled and should not be
 *						passed to a higher level handler.
 *
 ***********************************************************************/
static Boolean ApplicationHandleEvent(EventPtr event)
{
	FormPtr	frm;
	Word		formId;
	Boolean	handled = false;

	if (event->eType == frmLoadEvent)
		{
		// Load the form resource specified in the event then activate the form.
		formId = event->data.frmLoad.formID;
		frm = FrmInitForm(formId);
		FrmSetActiveForm(frm);

		// Set the event handler for the form.  The handler of the currently 
		// active form is called by FrmDispatchEvent each time it receives an event.
		switch (formId)
			{
			case CounterMainForm:
				FrmSetEventHandler(frm, MainFormHandleEvent);
				break;
			case SetNumberForm:
				FrmSetEventHandler(frm, SetNumberHandleEvent);
				break;
			case PrefsForm:
				FrmSetEventHandler(frm, PrefrenceHandleEvent);
				break;
			}
		handled = true;
		}
	
	return handled;
}

/***********************************************************************
 *
 * FUNCTION:		EventLoop
 *
 * DESCRIPTION:	A simple loop that obtains events from the Event
 *						Manager and passes them on to various applications and
 *						system event handlers before passing them on to
 *						FrmDispatchEvent for default processing.
 *
 * PARAMETERS:		None.
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void EventLoop(void)
{
	EventType	event;
	Word			error;
	
	do
		{
		// Get the next available event.
		EvtGetEvent(&event, evtWaitForever);
		
		// Give the system a chance to handle the event.
		if (! SysHandleEvent(&event))

			// P2. Give the menu bar a chance to update and handle the event.	
			if (! MenuHandleEvent(0, &event, &error))

				// P3. Give the application a chance to handle the event.
				if (! ApplicationHandleEvent(&event))

					// P3. Let the form object provide default handling of the event.
					FrmDispatchEvent(&event);
		}
	while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:		PilotMain
 *
 * DESCRIPTION:	This function is the equivalent of a main() function
 *						in standard C.  It is called by the Emulator to begin
 *						execution of this application.
 *
 * PARAMETERS:		cmd - command specifying how to launch the application.
 *						cmdPBP - parameter block for the command.
 *						launchFlags - flags used to configure the launch.			
 *
 * RETURNED:		Any applicable error code.
 *
 ***********************************************************************/
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
	// Check for a normal launch.
	if (cmd == sysAppLaunchCmdNormalLaunch)
		{
		// Set up initial form.
		StartApplication();
			
		// Start up the event loop.
		EventLoop();
		
		// Stop app
		StopApplication();
		}
	
	return(0);
}
